@@ -4,15 +4,16 @@ from typing import Mapping, Optional, Type
 
 import aiohttp
 
+from neuro_sdk.service_accounts import ServiceAccounts
+
 from .admin import _Admin
-from .blob_storage import BlobStorage
+from .buckets import Buckets
 from .config import Config
 from .core import _Core
 from .disks import Disks
 from .images import Images
 from .jobs import Jobs
 from .parser import Parser
-from .quota import _Quota
 from .secrets import Secrets
 from .server_cfg import Preset
 from .storage import Storage
@@ -43,12 +44,12 @@ class Client(metaclass=NoPublicConstructor):
         self._parser = Parser._create(self._config)
         self._admin = _Admin._create(self._core, self._config)
         self._jobs = Jobs._create(self._core, self._config, self._parser)
-        self._blob_storage = BlobStorage._create(self._core, self._config)
         self._storage = Storage._create(self._core, self._config)
-        self._quota = _Quota._create(self._core, self._config)
         self._users = Users._create(self._core, self._config)
         self._secrets = Secrets._create(self._core, self._config)
         self._disks = Disks._create(self._core, self._config)
+        self._service_accounts = ServiceAccounts._create(self._core, self._config)
+        self._buckets = Buckets._create(self._core, self._config, self._parser)
         self._images: Optional[Images] = None
 
     async def close(self) -> None:
@@ -95,10 +96,6 @@ class Client(metaclass=NoPublicConstructor):
     def jobs(self) -> Jobs:
         return self._jobs
 
-    @property
-    def blob_storage(self) -> BlobStorage:
-        return self._blob_storage
-
     @property
     def storage(self) -> Storage:
         return self._storage
@@ -121,6 +118,14 @@ class Client(metaclass=NoPublicConstructor):
     def disks(self) -> Disks:
         return self._disks
 
+    @property
+    def service_accounts(self) -> ServiceAccounts:
+        return self._service_accounts
+
+    @property
+    def buckets(self) -> Buckets:
+        return self._buckets
+
     @property
     def parse(self) -> Parser:
         return self._parser
