@@ -7,6 +7,7 @@ import logging
 import os
 import re
 import time
+import warnings
 from dataclasses import dataclass
 from email.utils import parsedate_to_datetime
 from pathlib import Path
@@ -14,6 +15,7 @@ from stat import S_ISREG
 from typing import (
     AbstractSet,
     Any,
+    AsyncContextManager,
     AsyncIterator,
     Awaitable,
     Callable,
@@ -47,15 +49,22 @@ from .abc import (
 )
 from .config import Config
 from .core import _Core
-from .errors import ResourceNotFound
-from .file_filter import FileFilter
+from .errors import NDJSONError, ResourceNotFound
+from .file_filter import AsyncFilterFunc, FileFilter
 from .url_utils import (
     _extract_path,
     normalize_local_path_uri,
     normalize_storage_path_uri,
 )
 from .users import Action
-from .utils import NoPublicConstructor, QueuedCall, queue_calls, retries
+from .utils import (
+    NoPublicConstructor,
+    QueuedCall,
+    aclosing,
+    asyncgeneratorcontextmanager,
+    queue_calls,
+    retries,
+)
 
 log = logging.getLogger(__name__)
 
@@ -91,6 +100,14 @@ class FileStatus:
         return Path(self.path).name
 
 
+@dataclass(frozen=True)
+class DiskUsageInfo:
+    cluster_name: str
+    total: int
+    used: int
+    free: int
+
+
 class Storage(metaclass=NoPublicConstructor):
     def __init__(self, core: _Core, config: Config) -> None:
         self._core = core
@@ -162,7 +179,8 @@ class Storage(metaclass=NoPublicConstructor):
                 return local.st_size
         return 0
 
-    async def ls(self, uri: URL) -> AsyncIterator[FileStatus]:
+    @asyncgeneratorcontextmanager
+    async def list(self, uri: URL) -> AsyncIterator[FileStatus]:
         uri = self._normalize_uri(uri)
         url = self._get_storage_url(uri, normalized=True)
         url = url.with_query(op="LISTSTATUS")
@@ -185,6 +203,16 @@ class Storage(metaclass=NoPublicConstructor):
                 for status in res["FileStatuses"]["FileStatus"]:
                     yield _file_status_from_api_ls(uri, status)
 
+    def ls(self, uri: URL) -> AsyncContextManager[AsyncIterator[FileStatus]]:
+        warnings.warn(
+            "client.storage.ls() ls is deprecated and scheduled for removal "
+            "in future Neuro SDK release, please use client.storage.list() instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return self.list(uri)
+
+    @asyncgeneratorcontextmanager
     async def glob(self, uri: URL, *, dironly: bool = False) -> AsyncIterator[URL]:
         if not _has_magic(uri.path):
             yield uri
@@ -197,27 +225,31 @@ class Storage(metaclass=NoPublicConstructor):
             glob_in_dir = self._glob1
         else:
             glob_in_dir = self._glob2
-        async for parent in self.glob(uri.parent, dironly=True):
-            async for x in glob_in_dir(parent, basename, dironly):
-                yield x
+        async with self.glob(uri.parent, dironly=True) as parent_iter:
+            async for parent in parent_iter:
+                async with aclosing(glob_in_dir(parent, basename, dironly)) as it:
+                    async for x in it:
+                        yield x
 
     async def _glob2(
         self, parent: URL, pattern: str, dironly: bool
     ) -> AsyncIterator[URL]:
         assert _isrecursive(pattern)
         yield parent
-        async for x in self._rlistdir(parent, dironly):
-            yield x
+        async with aclosing(self._rlistdir(parent, dironly)) as it:
+            async for x in it:
+                yield x
 
     async def _glob1(
         self, parent: URL, pattern: str, dironly: bool
     ) -> AsyncIterator[URL]:
         allow_hidden = _ishidden(pattern)
         match = re.compile(fnmatch.translate(pattern)).fullmatch
-        async for stat in self._iterdir(parent, dironly):
-            name = stat.path
-            if (allow_hidden or not _ishidden(name)) and match(name):
-                yield parent / name
+        async with aclosing(self._iterdir(parent, dironly)) as it:
+            async for stat in it:
+                name = stat.path
+                if (allow_hidden or not _ishidden(name)) and match(name):
+                    yield parent / name
 
     async def _glob0(
         self, parent: URL, basename: str, dironly: bool
@@ -230,19 +262,22 @@ class Storage(metaclass=NoPublicConstructor):
         yield uri
 
     async def _iterdir(self, uri: URL, dironly: bool) -> AsyncIterator[FileStatus]:
-        async for stat in self.ls(uri):
-            if not dironly or stat.is_dir():
-                yield stat
+        async with self.list(uri) as it:
+            async for stat in it:
+                if not dironly or stat.is_dir():
+                    yield stat
 
     async def _rlistdir(self, uri: URL, dironly: bool) -> AsyncIterator[URL]:
-        async for stat in self._iterdir(uri, dironly):
-            name = stat.path
-            if not _ishidden(name):
-                x = uri / name
-                yield x
-                if stat.is_dir():
-                    async for y in self._rlistdir(x, dironly):
-                        yield y
+        async with aclosing(self._iterdir(uri, dironly)) as it:
+            async for stat in it:
+                name = stat.path
+                if not _ishidden(name):
+                    x = uri / name
+                    yield x
+                    if stat.is_dir():
+                        async with aclosing(self._rlistdir(x, dironly)) as it2:
+                            async for y in it2:
+                                yield y
 
     async def mkdir(
         self, uri: URL, *, parents: bool = False, exist_ok: bool = False
@@ -315,6 +350,23 @@ class Storage(metaclass=NoPublicConstructor):
             res = await resp.json()
             return _file_status_from_api_stat(uri.host, res["FileStatus"])
 
+    async def disk_usage(self, cluster_name: Optional[str] = None) -> DiskUsageInfo:
+        cluster_name = cluster_name or self._config.cluster_name
+        uri = self._normalize_uri(
+            URL(f"storage://{cluster_name}/{self._config.username}")
+        )
+        assert uri.host is not None
+        url = self._get_storage_url(uri, normalized=True)
+        url = url.with_query(op="GETDISKUSAGE")
+        auth = await self._config._api_auth()
+
+        request_time = time.time()
+        async with self._core.request("GET", url, auth=auth) as resp:
+            self._set_time_diff(request_time, resp)
+            res = await resp.json()
+            return _disk_usage_from_api(uri.host, res)
+
+    @asyncgeneratorcontextmanager
     async def open(
         self, uri: URL, offset: int = 0, size: Optional[int] = None
     ) -> AsyncIterator[bytes]:
@@ -383,7 +435,7 @@ class Storage(metaclass=NoPublicConstructor):
             if os_errno is not None:
                 os_errno = errno.__dict__.get(os_errno, os_errno)
                 raise OSError(os_errno, err_text)
-            raise Exception(err_text)
+            raise NDJSONError(err_text)
 
     async def _rm(
         self, uri: URL, *, recursive: bool, progress: _AsyncAbstractDeleteProgress
@@ -537,12 +589,10 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool = False,
         continue_: bool = False,
-        filter: Optional[Callable[[str], Awaitable[bool]]] = None,
+        filter: Optional[AsyncFilterFunc] = None,
         ignore_file_names: AbstractSet[str] = frozenset(),
         progress: Optional[AbstractRecursiveFileProgress] = None,
     ) -> None:
-        if filter is None:
-            filter = _always
         src = normalize_local_path_uri(src)
         dst = normalize_storage_path_uri(
             dst, self._config.username, self._config.cluster_name
@@ -552,6 +602,12 @@ class Storage(metaclass=NoPublicConstructor):
             raise FileNotFoundError(errno.ENOENT, "No such file", str(path))
         if not path.is_dir():
             raise NotADirectoryError(errno.ENOTDIR, "Not a directory", str(path))
+
+        if filter is None:
+            filter = _always
+        if ignore_file_names:
+            filter = load_parent_ignore_files(filter, ignore_file_names, path)
+
         async_progress: _AsyncAbstractRecursiveFileProgress
         queue, async_progress = queue_calls(progress)
         await run_progress(
@@ -578,7 +634,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool,
         continue_: bool,
-        filter: Callable[[str], Awaitable[bool]],
+        filter: AsyncFilterFunc,
         ignore_file_names: AbstractSet[str],
         progress: _AsyncAbstractRecursiveFileProgress,
     ) -> None:
@@ -589,11 +645,12 @@ class Storage(metaclass=NoPublicConstructor):
                 try:
                     for retry in retries(f"Fail to list {dst}"):
                         async with retry:
-                            dst_files = {
-                                item.name: item
-                                async for item in self.ls(dst)
-                                if item.is_file()
-                            }
+                            async with self.list(dst) as it:
+                                dst_files = {
+                                    item.name: item
+                                    async for item in it
+                                    if item.is_file()
+                                }
                     exists = True
                 except ResourceNotFound:
                     update = continue_ = False
@@ -725,15 +782,15 @@ class Storage(metaclass=NoPublicConstructor):
                     if pos >= size:
                         break
                     async with retry:
-                        it = self.open(src, offset=pos)
-                        async for chunk in it:
-                            pos += len(chunk)
-                            await progress.step(
-                                StorageProgressStep(src, dst, pos, size)
-                            )
-                            await loop.run_in_executor(None, stream.write, chunk)
-                            if chunk:
-                                retry.reset()
+                        async with self.open(src, offset=pos) as it:
+                            async for chunk in it:
+                                pos += len(chunk)
+                                await progress.step(
+                                    StorageProgressStep(src, dst, pos, size)
+                                )
+                                await loop.run_in_executor(None, stream.write, chunk)
+                                if chunk:
+                                    retry.reset()
 
             await progress.complete(StorageProgressComplete(src, dst, size))
 
@@ -744,7 +801,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool = False,
         continue_: bool = False,
-        filter: Optional[Callable[[str], Awaitable[bool]]] = None,
+        filter: Optional[AsyncFilterFunc] = None,
         progress: Optional[AbstractRecursiveFileProgress] = None,
     ) -> None:
         if filter is None:
@@ -780,7 +837,7 @@ class Storage(metaclass=NoPublicConstructor):
         *,
         update: bool,
         continue_: bool,
-        filter: Callable[[str], Awaitable[bool]],
+        filter: AsyncFilterFunc,
         progress: _AsyncAbstractRecursiveFileProgress,
     ) -> None:
         dst_path.mkdir(parents=True, exist_ok=True)
@@ -798,7 +855,8 @@ class Storage(metaclass=NoPublicConstructor):
 
         for retry in retries(f"Fail to list {src}"):
             async with retry:
-                folder = [item async for item in self.ls(src)]
+                async with self.list(src) as it:
+                    folder = [item async for item in it]
 
         for child in folder:
             name = child.name
@@ -889,6 +947,15 @@ def _file_status_from_api_stat(cluster_name: str, values: Dict[str, Any]) -> Fil
     )
 
 
+def _disk_usage_from_api(cluster_name: str, values: Dict[str, Any]) -> DiskUsageInfo:
+    return DiskUsageInfo(
+        cluster_name=cluster_name,
+        total=values["total"],
+        used=values["used"],
+        free=values["free"],
+    )
+
+
 def _parse_content_range(rng_str: Optional[str]) -> slice:
     if rng_str is None:
         raise RuntimeError("Missed header Content-Range")
@@ -945,3 +1012,24 @@ async def run_concurrently(coros: Iterable[Awaitable[Any]]) -> None:
 
 async def _always(path: str) -> bool:
     return True
+
+
+def load_parent_ignore_files(
+    filter: AsyncFilterFunc,
+    ignore_file_names: AbstractSet[str],
+    path: Path,
+    rel_path: str = "",
+) -> AsyncFilterFunc:
+    if path == path.parent:
+        return filter
+    rel_path = f"{path.name}/{rel_path}"
+    path = path.parent
+    filter = load_parent_ignore_files(filter, ignore_file_names, path, rel_path)
+    for name in ignore_file_names:
+        config_path = path / name
+        if config_path.exists():
+            log.debug(f"Load ignore file {str(config_path)!r}")
+            file_filter = FileFilter(filter)
+            file_filter.read_from_file(config_path, "", rel_path)
+            filter = file_filter.match
+    return filter
