@@ -135,11 +135,11 @@ def multiple_clusters_config() -> Dict[str, Cluster]:
             name="default",
             registry_url=URL("https://registry-dev.neu.ro"),
             storage_url=URL("https://storage-dev.neu.ro"),
-            blob_storage_url=URL("https://blob-storage-dev.neu.ro"),
             users_url=URL("https://users-dev.neu.ro"),
             monitoring_url=URL("https://jobs-dev.neu.ro"),
             secrets_url=URL("https://secrets-dev.neu.ro"),
             disks_url=URL("https://disks-dev.neu.ro"),
+            buckets_url=URL("https://buckets-dev.neu.ro"),
             presets={
                 "cpu-small": Preset(
                     credits_per_hour=Decimal("10"), cpu=1, memory_mb=2 * 1024
@@ -150,11 +150,11 @@ def multiple_clusters_config() -> Dict[str, Cluster]:
             name="another",
             registry_url=URL("https://registry2-dev.neu.ro"),
             storage_url=URL("https://storage2-dev.neu.ro"),
-            blob_storage_url=URL("https://blob-storage-dev.neu.ro"),
             users_url=URL("https://users2-dev.neu.ro"),
             monitoring_url=URL("https://jobs2-dev.neu.ro"),
             secrets_url=URL("https://secrets2-dev.neu.ro"),
-            disks_url=URL("https://disks-dev.neu.ro"),
+            disks_url=URL("https://disks2-dev.neu.ro"),
+            buckets_url=URL("https://buckets2-dev.neu.ro"),
             presets={
                 "cpu-large": Preset(
                     credits_per_hour=Decimal("10"), cpu=7, memory_mb=14 * 1024
@@ -180,9 +180,10 @@ async def test_get_cluster_name_from_local(
         assert client.config.cluster_name == "default"
         assert client.config.registry_url == URL("https://registry-dev.neu.ro")
         assert client.config.storage_url == URL("https://storage-dev.neu.ro")
-        assert client.config.blob_storage_url == URL("https://blob-storage-dev.neu.ro")
         assert client.config.monitoring_url == URL("https://jobs-dev.neu.ro")
         assert client.config.secrets_url == URL("https://secrets-dev.neu.ro")
+        assert client.config.disk_api_url == URL("https://disks-dev.neu.ro")
+        assert client.config.bucket_api_url == URL("https://buckets-dev.neu.ro")
         assert client.config.presets == {
             "cpu-small": Preset(
                 credits_per_hour=Decimal("10"), cpu=1, memory_mb=2 * 1024
@@ -196,6 +197,8 @@ async def test_get_cluster_name_from_local(
         assert client.config.storage_url == URL("https://storage2-dev.neu.ro")
         assert client.config.monitoring_url == URL("https://jobs2-dev.neu.ro")
         assert client.config.secrets_url == URL("https://secrets2-dev.neu.ro")
+        assert client.config.disk_api_url == URL("https://disks2-dev.neu.ro")
+        assert client.config.bucket_api_url == URL("https://buckets2-dev.neu.ro")
         assert client.config.presets == {
             "cpu-large": Preset(
                 credits_per_hour=Decimal("10"), cpu=7, memory_mb=14 * 1024
@@ -315,22 +318,22 @@ async def test_clusters(
                 name="default",
                 registry_url=URL("https://registry-dev.neu.ro"),
                 storage_url=srv.make_url("/storage"),
-                blob_storage_url=srv.make_url("/blob"),
                 users_url=srv.make_url("/"),
                 monitoring_url=srv.make_url("/jobs"),
                 secrets_url=srv.make_url("/secrets"),
                 disks_url=srv.make_url("/disk"),
+                buckets_url=srv.make_url("/buckets"),
                 presets=mock.ANY,
             ),
             "another": Cluster(
                 name="another",
                 registry_url=srv.make_url("/registry2"),
                 storage_url=srv.make_url("/storage2"),
-                blob_storage_url=srv.make_url("/blob2"),
                 users_url=srv.make_url("/"),
                 monitoring_url=srv.make_url("/jobs2"),
                 secrets_url=srv.make_url("/secrets2"),
                 disks_url=srv.make_url("/disk2"),
+                buckets_url=srv.make_url("/buckets2"),
                 presets=mock.ANY,
             ),
         }
@@ -342,11 +345,11 @@ async def test_fetch(
     admin_url = "https://admin-dev.neu.ro"
     registry_url = "https://registry2-dev.neu.ro"
     storage_url = "https://storage2-dev.neu.ro"
-    blob_storage_url = "https://blob-storage2-dev.neu.ro"
     users_url = "https://users2-dev.neu.ro"
     monitoring_url = "https://jobs2-dev.neu.ro"
     secrets_url = "https://secrets2-dev.neu.ro"
     disks_url = "https://disks2-dev.neu.ro"
+    buckets_url = "https://buckets2-dev.neu.ro"
     auth_url = "https://dev-neuro.auth0.com/authorize"
     token_url = "https://dev-neuro.auth0.com/oauth/token"
     logout_url = "https://dev-neuro.auth0.com/v2/logout"
@@ -368,11 +371,11 @@ async def test_fetch(
                 "name": "default",
                 "registry_url": registry_url,
                 "storage_url": storage_url,
-                "blob_storage_url": blob_storage_url,
                 "users_url": users_url,
                 "monitoring_url": monitoring_url,
                 "secrets_url": secrets_url,
                 "disks_url": disks_url,
+                "buckets_url": buckets_url,
                 "resource_presets": [
                     {
                         "name": "cpu-small",
@@ -399,11 +402,11 @@ async def test_fetch(
                 name="default",
                 registry_url=URL(registry_url),
                 storage_url=URL(storage_url),
-                blob_storage_url=URL(blob_storage_url),
                 users_url=URL(users_url),
                 monitoring_url=URL(monitoring_url),
                 secrets_url=URL(secrets_url),
                 disks_url=URL(disks_url),
+                buckets_url=URL(buckets_url),
                 presets={
                     "cpu-small": Preset(
                         credits_per_hour=Decimal("10"),
@@ -427,11 +430,11 @@ async def test_fetch_dropped_selected_cluster(
     admin_url = "https://admin-dev.neu.ro"
     registry_url = "https://registry2-dev.neu.ro"
     storage_url = "https://storage2-dev.neu.ro"
-    blob_storage_url = "https://blob-storage2-dev.neu.ro"
     users_url = "https://users2-dev.neu.ro"
     monitoring_url = "https://jobs2-dev.neu.ro"
     secrets_url = "https://secrets2-dev.neu.ro"
     disks_url = "https://disks2-dev.neu.ro"
+    buckets_url = "https://buckets2-dev.neu.ro"
     auth_url = "https://dev-neuro.auth0.com/authorize"
     token_url = "https://dev-neuro.auth0.com/oauth/token"
     logout_url = "https://dev-neuro.auth0.com/v2/logout"
@@ -453,11 +456,11 @@ async def test_fetch_dropped_selected_cluster(
                 "name": "another",
                 "registry_url": registry_url,
                 "storage_url": storage_url,
-                "blob_storage_url": blob_storage_url,
                 "users_url": users_url,
                 "monitoring_url": monitoring_url,
                 "secrets_url": secrets_url,
                 "disks_url": disks_url,
+                "buckets_url": buckets_url,
                 "resource_presets": [
                     {
                         "name": "cpu-small",
@@ -530,11 +533,11 @@ async def test_check_server_mismatch_clusters(
     admin_url = "https://admin-dev.neu.ro"
     registry_url = "https://registry2-dev.neu.ro"
     storage_url = "https://storage2-dev.neu.ro"
-    blob_storage_url = "https://blob-storage2-dev.neu.ro"
     users_url = "https://users2-dev.neu.ro"
     monitoring_url = "https://jobs2-dev.neu.ro"
     secrets_url = "https://secrets2-dev.neu.ro"
     disks_url = "https://disks2-dev.neu.ro"
+    buckets_url = "https://buckets2-dev.neu.ro"
     auth_url = "https://dev-neuro.auth0.com/authorize"
     token_url = "https://dev-neuro.auth0.com/oauth/token"
     logout_url = "https://dev-neuro.auth0.com/v2/logout"
@@ -556,11 +559,11 @@ async def test_check_server_mismatch_clusters(
                 "name": "another",
                 "registry_url": registry_url,
                 "storage_url": storage_url,
-                "blob_storage_url": blob_storage_url,
                 "users_url": users_url,
                 "monitoring_url": monitoring_url,
                 "secrets_url": secrets_url,
                 "disks_url": disks_url,
+                "buckets_url": buckets_url,
                 "resource_presets": [
                     {
                         "name": "cpu-small",
@@ -597,11 +600,11 @@ async def test_check_server_mismatch_auth(
     admin_url = "https://admin-dev.neu.ro"
     registry_url = "https://registry2-dev.neu.ro"
     storage_url = "https://storage2-dev.neu.ro"
-    blob_storage_url = "https://blob-storage2-dev.neu.ro"
     users_url = "https://users2-dev.neu.ro"
     monitoring_url = "https://jobs2-dev.neu.ro"
     secrets_url = "https://secrets2-dev.neu.ro"
     disks_url = "https://disks2-dev.neu.ro"
+    buckets_url = "https://buckets2-dev.neu.ro"
     auth_url = "https://dev-neuro.auth0.com/authorize"
     token_url = "https://dev-neuro.auth0.com/oauth/token"
     logout_url = "https://dev-neuro.auth0.com/v2/logout"
@@ -622,11 +625,11 @@ async def test_check_server_mismatch_auth(
                 "name": "default",
                 "registry_url": registry_url,
                 "storage_url": storage_url,
-                "blob_storage_url": blob_storage_url,
                 "users_url": users_url,
                 "monitoring_url": monitoring_url,
                 "secrets_url": secrets_url,
                 "disks_url": disks_url,
+                "buckets_url": buckets_url,
                 "resource_presets": [
                     {
                         "name": "cpu-small",
